from rest_framework import viewsets, permissions, status
from rest_framework.response import Response
from rest_framework.decorators import action

from django.views.generic import DetailView
from django.urls import resolve
from django.http import Http404

from courses.permissions import IsProfessorCoordinatorOrAdminPermissionOrReadOnly

from courses.certification.serializers import (
    CertificationProcessSerializer,
    CourseCertificationSerializer,
    CertificateTemplateSerializer,
    CertificateTemplateCreateSerializer,
    CertificateTemplateImageSerializer,
)

from courses.certification.models import (
    CertificationProcess,
    CourseCertification,
    CertificateTemplate,
)

from courses.models import Course
from courses.workspaces.models import Workspace


class CertificateDataMixin(viewsets.ModelViewSet):
     def get_queryset(self):
        queryset = CertificateTemplate.objects.all()
        course = self.request.query_params.get('course', None)
        if course:
            queryset = queryset.filter(course=course)

        workspace = self.request.query_params.get('workspace', None)
        if workspace:
            queryset = queryset.filter(workspace=workspace)

        return queryset


class CertificateDataViewSet(CertificateDataMixin, viewsets.ModelViewSet):
    model = CertificateTemplate
    permission_classes = (IsProfessorCoordinatorOrAdminPermissionOrReadOnly, )

    def get_serializer_class(self):
        if self.action == 'create':
            return CertificateTemplateCreateSerializer
        return CertificateTemplateSerializer

    @action(detail=True, methods=['get', 'post'])
    def images(self, request, pk=None):
        obj = self.get_object()
        errors = []

        clear_logos = (
            ('cert_logo', request.data.get('cert_logo_clear', None)),
            ('base_logo', request.data.get('base_logo_clear', None)),
            ('signature', request.data.get('signature_clear', None)),
            ('site_logo', request.data.get('site_logo_clear', None)),
        )
        for cl in clear_logos:
            if(cl[1]):
                setattr(obj, cl[0], None)

        serializer = CertificateTemplateImageSerializer(obj, request.FILES)
        if serializer.is_valid():
            serializer.save()
            return Response(CertificateTemplateSerializer(obj).data,
                            status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class CertificationProcessViewSet(viewsets.ModelViewSet):
    model = CertificationProcess
    filter_fields = ('student',)
    serializer_class = CertificationProcessSerializer

    permission_classes = []

    def get_queryset(self):
        queryset = super().get_queryset()

        klass = self.request.query_params.get('klass')
        if klass:
            queryset = queryset.filter(klass=klass)

        return queryset


class CourseCertificationViewSet(viewsets.ModelViewSet):
    queryset = CourseCertification.objects.all()
    model = CourseCertification
    lookup_field = 'link_hash'
    filter_fields = ('course_student',)
    serializer_class = CourseCertificationSerializer

    def get_queryset(self):
        queryset = super(CourseCertificationViewSet, self).get_queryset()
        if not self.request.GET.get('user', False):
            queryset = queryset.filter(course_student__user=self.request.user)

        cert = queryset.first()
        cs = cert.course_student

        # Change receipt into certificate
        if cs.get_current_class().user_can_certificate and cs.course_finished or \
           cs.get_current_class().user_can_certificate_even_without_progress:
            cert.type = 'certificate'
            cert.save()

        return queryset


class CertificateTemplateViewSet(viewsets.ModelViewSet):
    model = CertificateTemplate
    queryset = CertificateTemplate.objects.all()
    #lookup_field = 'course'
    permission_classes = (IsProfessorCoordinatorOrAdminPermissionOrReadOnly, )
    serializer_class = CertificateTemplateSerializer

    def get_serializer_class(self):
        if self.action == 'create':
            return CertificateTemplateCreateSerializer
        return CertificateTemplateSerializer

    @action(detail=True, methods=['get', 'post'])
    def images(self, request, pk=None):
        obj = self.get_object()
        errors = []

        clear_logos = (
            ('cert_logo', request.data.get('cert_logo_clear', None)),
            ('base_logo', request.data.get('base_logo_clear', None)),
            ('signature', request.data.get('signature_clear', None)),
            ('second_signature', request.data.get('second_signature_clear', None)),
            ('site_logo', request.data.get('site_logo_clear', None)),
        )
        for cl in clear_logos:
            if(cl[1]):
                setattr(obj, cl[0], None)

        serializer = CertificateTemplateImageSerializer(obj, request.FILES)
        if serializer.is_valid():
            serializer.save()
            return Response(CertificateTemplateSerializer(obj).data,
                            status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def update(self, request, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data)
        if serializer.is_valid():
            serializer.save()
        return Response(serializer.data)


class CertificateTemplateImageViewSet(viewsets.ModelViewSet):
    queryset = CertificateTemplate.objects.all()
    model = CertificateTemplate
    lookup_field = 'course'
    serializer_class = CertificateTemplateImageSerializer
    permission_classes = (IsProfessorCoordinatorOrAdminPermissionOrReadOnly, )

    def post(self, request, **kwargs):
        certificate_template = self.get_object()
        serializer = CertificateTemplateImageSerializer(certificate_template,
                                                        request.FILES)

        if serializer.is_valid():
            serializer.save()
            return Response(status=200)
        else:
            return Response(serializer.errors, status=400)


class CourseCertificationDetailView(DetailView):
    model = CourseCertification
    template_name = 'certificate.html'
    slug_field = "link_hash"
    serializer_class = CourseCertificationSerializer

    def render_to_response(self, context, **response_kwargs):

        certificate = context.get('object')
        workspace = certificate.course_student\
            .get_current_class().contract.first()

        if not certificate.course_student.can_emmit_receipt():
            raise Http404

        if certificate:
            context['cert_template'] = CertificateTemplate.objects.get(
                course=certificate.course_student.course,
                document_type=certificate.type,
            )
            # Interpolate data into text string
            # {Nome} : certificate.student (name)
            # {CPF} : certificate.student.cpf)
            # {MODULO} : certificate.course.name)
            # {CONTRATO} : context['cert_template'].workspace.name)
            # {NUM_UNIDADES} : str(certificate.course_total_units))
            # {HORAS} : str(certificate.course.course_load)
            # {TURMA} : certificate.course_student.get_current_class().name)
            # {DATA_INICIO} : certificate.course_student.start_date
            # {DATA_FIM} : certificate.course_student.finish_date

            if '{MODULO}' in context['cert_template'].text:
                context['cert_template'].text = context['cert_template'].text.replace(
                    '{MODULO}',
                    certificate.course.name,
                )

            if '{CPF}' in context['cert_template'].text:
                context['cert_template'].text = context['cert_template'].text.replace(
                    '{CPF}',
                    certificate.student.cpf,
                )

            # Prepare start and finish dates
            start_date = '{0}/{1}/{2}'.format(
                certificate.course_student.start_date.day,
                certificate.course_student.start_date.month,
                certificate.course_student.start_date.year
            )

            finish_date = '{0}/{1}/{2}'.format(
                certificate.course_student.finish_date.day,
                certificate.course_student.finish_date.month,
                certificate.course_student.finish_date.year
            ) if certificate.course_student.finish_date else 'xx/xx/xxxx'

            workspace = context['cert_template'].workspace
            w_name = workspace.name if workspace else ''
            context['cert_template'].text = context['cert_template'].text\
                .replace('{NOME}', certificate.student.get_full_name())\
                .replace('{CONTRATO}', w_name)\
                .replace('{NUM_UNIDADES}', str(certificate.course_total_units))\
                .replace('{HORAS}', str(certificate.course.course_load))\
                .replace('{TURMA}', certificate.course_student.get_current_class().name)\
                .replace('{DATA_INICIO}', start_date)\
                .replace('{DATA_FIM}', finish_date)

        url_name = resolve(self.request.path_info).url_name

        return super().render_to_response(context, **response_kwargs)
