from actstream import action
from actstream.models import user_stream, Action, actor_stream

from rest_framework import views, viewsets, mixins, permissions, status
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated
from rest_framework.pagination import PageNumberPagination

from rest_pandas import PandasViewSet
from rest_pandas.renderers import PandasCSVRenderer, PandasJSONRenderer, PandasExcelRenderer
import pandas as pd

from discussion.models import Forum, Topic
from .serializers import UserActionsSerializer, UserAccessSerializer, ActionSerializer
from .models import AccessibleArea
from ..models import Course
from ..classroom.models import Classroom

from cards.models import Card
from courses.models import Unit

from courses_learning_objects.models import LearningObject

from django.contrib.auth.models import User
from django.contrib.auth import get_user_model

User = get_user_model()


class UserActionsView(views.APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request, format=None):
        # Only models from this list are registered to be used on actions
        allowed_models = {
            'Course': Course,
            'Classroom': Classroom,
            'Topic': Topic,
            'Forum': Forum,
            'AccessibleArea': AccessibleArea,
            'LearningObject': LearningObject,
            'Card': Card,
            'Unit': Unit,
        }

        # Validate incoming data
        serializer = UserActionsSerializer(data=request.data)
        if not serializer.is_valid():
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        data = serializer.data

        # Get the actual model objects for action and target
        action_object = None
        target = None
        if 'action_object_type' in data and 'action_object_id' in data:
            action_object = allowed_models[data['action_object_type']].objects.get(id=data['action_object_id'])
        if 'target_type' in data and 'target_id' in data:
            target = allowed_models[data['target_type']].objects.get(id=data['target_id'])

        # Save the action
        action.send(request.user, verb=data['verb'], action_object=action_object, target=target)

        return Response(None, status=status.HTTP_201_CREATED)


class UserAccessView(views.APIView):
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request, format=None):

        # Validate incoming data
        serializer = UserAccessSerializer(data=request.data)
        if not serializer.is_valid():
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        data = serializer.data

        # Get the actual model object for action
        try:
            action_object = AccessibleArea.objects.get(slug=data['area'])
        except AccessibleArea.DoesNotExist:
            return Response({'error': 'Specified area not found'}, status=status.HTTP_404_NOT_FOUND)

        if 'target_type' in data and data['target_type'] == 'Classroom':
            target = Classroom.objects.get(id=data['target_id'])
        else:
            target = None

        # Save the action
        action.send(request.user, verb='access', action_object=action_object, target=target)

        return Response(None, status=status.HTTP_201_CREATED)


class ActivitiesPagination(PageNumberPagination):
    page_size = 30
    page_size_query_param = 'page_size'
    max_page_size = 100


class UserActivitiesViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = ActionSerializer
    permission_classes = [permissions.IsAuthenticated]
    pagination_class = ActivitiesPagination
    model = Action

    def get_queryset(self):
        user_id = self.request.GET.get('user')
        user = User.objects.get(id=user_id)

        queryset = actor_stream(user)

        return queryset


class UserActionsViewSet(PandasViewSet):
    permission_classes = [IsAuthenticated]
    renderer_classes = [
        PandasCSVRenderer,
        PandasJSONRenderer,
        PandasExcelRenderer,
    ]

    def get_queryset(self):
        user_id = self.request.GET.get('user')
        user = User.objects.get(id=user_id)

        queryset = actor_stream(user)

        return queryset

    def list(self, request, format=None):
        queryset = self.get_queryset()
        serializer = ActionSerializer(queryset, many=True)
        return self.update_pandas_headers(Response(self.transform_data(serializer.data)))

    def transform_data(self, data):
        response = []
        for user in data:
            compls = []

            phrase = user['action_phrase'].split()

            if user['complements']: 
                for v, k in user['complements'].items():
                    compls += [k]
            else:
                compls += [' '.join(phrase[2:]) , " ", " "]
            
            response.append({'Usuário': phrase[0], 'Ação': phrase[1], 'Espaço 1': compls[0], 'Espaço 2': compls[1], 'Espaço 3': compls[2], 'Data/Hora': user['timestamp']})

        return pd.DataFrame.from_dict(response)

    def get_pandas_filename(self, request, format):
        user_id = self.request.GET.get('user')
        user = User.objects.get(id=user_id)

        return 'Percursos do usuario - ' + user.name
